package zero1hd.polyjet.audio.map;

import org.apache.commons.math3.random.MersenneTwister;

import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.utils.FloatArray;

import zero1hd.polyjet.Polyjet;
import zero1hd.polyjet.audio.AudioAnalyzer;
import zero1hd.polyjet.entity.Entities;

public class RhythmMapAlgorithm implements Runnable {
	private FloatArray bassPeaks;
	private FloatArray UMPeaks;
	private FloatArray overlappedPeaks;
	private MersenneTwister rand;
	private GamePlayMap map;
	private float avgBPS;
	private float second;
	public RhythmMapAlgorithm(AudioAnalyzer analyzer) {
		bassPeaks = analyzer.getBassPeaks();
		UMPeaks = analyzer.getUMPeaks();
		overlappedPeaks = analyzer.getOverlappedPeaks();
		map = new GamePlayMap(analyzer.getAudioData());
		rand = new MersenneTwister(analyzer.getPUID());
		avgBPS = analyzer.getAvgBPS();
		second = analyzer.getsecondsPerFrame();
	}
	
	@Override
	public void run() {
		map.beginBuild();
		for (int index = 0; index < bassPeaks.size; index++) {
			if (bassPeaks.get(index) != 0 || UMPeaks.get(index) != 0) {
				if (overlappedPeaks.get(index+3) != 0) {
					//TODO basic void circle spawning
					float endRadius = rand.nextFloat()*Polyjet.GAME_AREA_WIDTH;
					
					map.addToMap(Entities.VOID_CIRCLE, 
							endRadius, 
							rand.nextFloat()*Polyjet.GAME_AREA_WIDTH, 
							rand.nextFloat()*Polyjet.GAME_AREA_HEIGHT,
							avgBPS,
							3f*second
							);
				}
				if (bassPeaks.get(index) != 0) {
					map.addToMap(Entities.BAR, 
							MathUtils.round(rand.nextFloat()*Polyjet.GAME_AREA_WIDTH), 
							bassPeaks.get(index)*Polyjet.GAME_AREA_HEIGHT/avgBPS);
				} else {
					if (UMPeaks.get(index) != 0) {
						//TODO basic pellet scatter spawn
					}
					//TODO rest of the basic generation
				}
			} else {
				map.addRestToMap();
			}
		}
		map.endBuild();
	}
	
	public synchronized GamePlayMap getMap() {
		return map;
	}
}
