package zero1hd.polyjet.ui.pages;

import com.badlogic.gdx.assets.AssetManager;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.math.Interpolation;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.scenes.scene2d.ui.Slider;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;
import com.badlogic.gdx.utils.Align;

import zero1hd.polyjet.audio.AudioAnalyzer;
import zero1hd.polyjet.audio.AudioData;
import zero1hd.polyjet.audio.AudioInfo;
import zero1hd.polyjet.ui.builders.ScrollText;
import zero1hd.polyjet.util.MiniEvents;
import zero1hd.polyjet.util.MiniListener;

public class AnalyzePage extends Page implements MiniListener {
	AudioAnalyzer audioAnalyzer;
	AudioData music;
	
	Table songInfo;
	Slider difficultyModifier;
	Label difficultyModifierPercentage;
	private Skin skin;
	
	private TextButton back;
	private Image loadingCircle;
	private long startTime, endTime;
	private ScrollText songName;
	private Label[] info;
	public AnalyzePage(Skin skin, AssetManager assets) {
		super("Results", skin);
		this.skin = skin;
		songInfo = new Table(skin);
		songInfo.setSize(getWidth()/2f, getHeightBelowTitle()/2f);
		songInfo.setBackground(skin.getDrawable("large-pane"));
		
		loadingCircle = new Image(assets.get("cybercircle1.png", Texture.class));
		loadingCircle.setPosition((getWidth()-loadingCircle.getWidth())/2, (getHeightBelowTitle()-loadingCircle.getHeight())/2);
		loadingCircle.setColor(0.8f,0.8f,0.8f,0.7f);
		loadingCircle.setOrigin(loadingCircle.getWidth()/2, loadingCircle.getHeight()/2);
		loadingCircle.addAction(Actions.forever(Actions.rotateBy(-360f, 4f)));
		addActor(loadingCircle);
		
		back = new TextButton("Back", skin);
		back.setPosition(getWidth()-back.getWidth()-15f, getHeightBelowTitle());
		back.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				miniSender.send(MiniEvents.BACK);
				audioAnalyzer.stop();
			}
		});
		
		info = new Label[6];
		for (int i = 0; i < info.length; i++) {
			info[i] = new Label(null, skin, "sub-font", skin.getColor("default"));
		}
		
		addActor(back);
		addActor(songInfo);
	}
	
	public void setSong(AudioData music, AudioInfo audioInfo, MiniListener listener) {
		songInfo.clear();
		songInfo.pad(15f);
		songInfo.defaults().align(Align.left);
		audioAnalyzer = new AudioAnalyzer();
		audioAnalyzer.sender.addListener(this);
		
		this.music = music;
		
		songName = new ScrollText(audioInfo.getSongName(), skin, true);
		audioAnalyzer.startAnalyticalThread(music);
		songInfo.add(songName).expandX().fillX();
		
		for (int i = 0; i < info.length; i++) {
			info[i].setColor(1f, 1f, 1f, 0f);
			info[i].setText(null);
			songInfo.row();
			songInfo.add(info[i]);
		}
		
		startTime = System.currentTimeMillis();
		songInfo.setPosition((getWidth()-songInfo.getWidth())/2f, (getHeightBelowTitle()-songInfo.getHeight())/2f);
		info[0].addAction(Actions.color(Color.BLACK, 2.5f));
		System.out.println(songInfo);
	}
	
	
	@Override
	public void act(float delta) {
		super.act(delta);
	}
	
	public boolean hasAudioData() {
		if (music != null) {
			return true;
		} else {
			return false;
		}
	}
	
	public void clearAudioData() {
		music = null;
	}

	@Override
	public void handle(MiniEvents ID) {
		switch (ID) {
		case ANALYZER_ITERATED:
			if (audioAnalyzer.getProgress() == 50) {
				songInfo.addAction(Actions.moveTo(40f, getHeightBelowTitle()-songInfo.getHeight()-25f, 0.75f, Interpolation.linear));
			}
			info[0].setText("Initial analysis: " + audioAnalyzer.getProgress() + "%");
			break;
		case SPECTRAL_FLUX_DONE:
			endTime = System.currentTimeMillis();
			info[1].setText("Done. Analyze time: " + ((endTime - startTime)/1000f) + "s");
			info[1].addAction(Actions.color(Color.BLACK, 0.75f));
			
			break;
		default:
			break;
		}
	}
}
